<?php

declare(strict_types=1);

namespace Sentry\Exception;

/**
 * This exception is thrown when a fatal error occurs.
 *
 * @author Stefano Arlandini <sarlandini@alice.it>
 */
final class FatalErrorException extends \ErrorException
{
}
